import {
  Config, Metadata, Loader, Http, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  Select, GridEditable, Form, Button, Modal, ModalService, SelectTree,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme11000PercentualAdesaoPratoController {
  private utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
  
  public gridEditableController = Loader.getInstance('GridEditableController');
  
  private modalRelatorioPercentAdesao!: Modal;
  
  public reportController = Loader.getInstance('ReportController');
  
  public filterTipoCardapio({ component }: IEventParam<Select>) {
    let unidade: Select | String;
    let cardapio: Select;
    let cdfilcentpme = this.appController.getCdCentral();
    let conditions;
    
    if (component.name === 'TPCARD_REPORT') {
      unidade = Metadata.getInstance<Select>('UNIDADE_AGREGADA_REPORT').value?.join('_');
      unidade = !unidade ? 'T' : unidade;
      cardapio = Metadata.getInstance<Select>('TPCARD_REPORT');
      const unidadeCentral = Metadata.getInstance<Select>('CDCENTRAL_REPORT');
      cdfilcentpme = unidadeCentral.value.length > 0 ? unidadeCentral.value.join('_') : 'T';
    
      conditions = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: unidade,
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: cdfilcentpme,
        },
      ];
    } else {
      unidade = Metadata.getInstance<Select>('UNIDADE_AGREGADA').value;
      unidade = !unidade ? '' : unidade;
      cardapio = Metadata.getInstance<Select>('TPCARD');
      
      conditions = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: unidade,
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: cdfilcentpme,
        },
        {
          name: 'NRPASTATCARD',
          operator: '=',
          value: 'T',
        },
      ];
      
      this.reloadPercAct();
    }
    
    cardapio.datasource.filter = conditions;
    cardapio.datasource.get();
  }
  
  private reloadPercAct() {
    const cardapio = Metadata.getInstance<Select>('TPCARD');
    if (cardapio.datasource.data.length > 0) {
      this.filterPercAceit(); //  cardapio.datasource.data[0] como argumento
    }
  }
  
  public filterAgregada({ component }: IEventParam<Select>) {
    const grid = Metadata.getInstance<GridEditable>('PERACEITPRATO');
    component.datasource.filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    component.datasource.get();
    grid.datasource.updateData([]);
    grid.datasource.total = 0;
  }
  
  public filterPratos({ component }: IEventParam<Select>) {
    component.datasource.filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: 'P',
      },
    ];
    
    component.datasource.get();
  }
  
  public changeUnidade({ component }: IEventParam<Select>) {
    let tipoCard:Select;
    if (component.name === 'UNIDADE_AGREGADA_REPORT') {
      tipoCard = Metadata.getInstance<Select>('TPCARD_REPORT');
    } else {
      tipoCard = Metadata.getInstance<Select>('TPCARD');
    }
    
    tipoCard.setValue('');
  }
  
  public reloadGridPerAceitaPrato({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formFilter');
    
    if (form.validate()) {
      this.gridEditableController.reload({ component });
    }
  }
  
  public verifyPratIni({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('formFilter');
    if (component.value !== null) {
      if (this.utilities.returnPrat(form.getChildInstance<Select>('PRATO_INICIAL').value, component.value)) {
        this.utilities.showAlert('Alert', 'pme::pratoIniMaior', 'error');
        component.setValue(null);
      }
      this.reloadPercAct();
    }
  }
  
  public controllPratoIni({ component }: IEventParam<Select>) {
    if (component.name === 'PRATO_FINAL') {
      const pratoFinal = Metadata.getInstance<Select>('PRATO_FINAL');
      if (pratoFinal.value !== null) pratoFinal.setValue(null);
      this.reloadPercAct();
    }
  }
  
  public filterPercAceit() {
    const unidade = Metadata.getInstance<Select>('UNIDADE_AGREGADA').value;
    const percPrato: GridEditable = Metadata.getInstance('PERACEITPRATO');
    const pratoIncial = Metadata.getInstance<Select>('PRATO_INICIAL').value;
    const pratoFinal = Metadata.getInstance<Select>('PRATO_FINAL').value;
    const tipoCard = Metadata.getInstance<Select>('TPCARD').selectValue;
    const nivelMaxPlate = (Config.masks.plateMask as PmeMask).nivelMax;
    const form = Metadata.getInstance<Form>('formFilter').value;
    if (form.TPCARD) {
      const conditions = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: unidade,
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'CDPRATOINI',
          operator: '=',
          value: pratoIncial !== null ? pratoIncial : '0',
        },
        {
          name: 'CDPRATOFIN',
          operator: '=',
          value: pratoFinal !== null ? `${pratoFinal}Z` : 'Z',
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: tipoCard.CDTIPOCARD,
        },
        {
          name: 'MAXCDNVPRATO',
          operator: '=',
          value: nivelMaxPlate,
        },
      ];
      percPrato.datasource.filter = conditions;
      percPrato.datasource.get();
    }
  }
  
  public dataSourceGet({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Form>('formFilter').value.UNIDADE_AGREGADA;

    component.datasource.filter = {
      CDFILIAL: unidade,
    };
  }
  
  public savePercAceita() {
    const grid: GridEditable = Metadata.getInstance('PERACEITPRATO');
    const editedRows = grid.getEditedRows();
    this.checkCentralNull(editedRows);
    
    if (editedRows.length > 0) {
      if (grid) grid.datasource.loading = true;
      Http.post('/percAceita/save', { requestType: 'Row', editedRows }).then((response) => {
        if (response.data.dataset.save[0]) {
          this.gridEditableController.cancelEditedRows({ component: grid });
          grid.datasource.get();
          this.utilities.showAlert('Alert', 'pme::salvoComSucesso', 'success');
        }
      }).finally(() => {
        if (grid) grid.datasource.loading = false;
      });
    }
  }
  
  private checkCentralNull(editedRows: IDictionary) {
    const centralNull = editedRows.filter((data: IDictionary) => data.CDFILCENTPME === null);
    
    if (centralNull.length > 0) {
      centralNull.forEach((row: IDictionary) => {
        row.CDFILCENTPME = this.appController.getCdCentral();
        row.CDFILIAL = Metadata.getInstance<Select>('UNIDADE_AGREGADA').value;
        row.__is_new = true;
      });
    }
  }
  
  public cancelEditingTelaPercentual({ component }: IDictionary) {
    this.gridEditableController.cancelEditing({ component });
  }
  
  private async createModalRelatorioPercentAdesao() {
    this.modalRelatorioPercentAdesao = await ModalService.createFromJson(
      'pme/parametrizacao/modals/pme11000_percent_adesao_relatorio', true,
    );
  }
  
  public openModalRelatorioPercentAdesao() {
    if (!this.modalRelatorioPercentAdesao) {
      this.createModalRelatorioPercentAdesao().then(() => {
        this.modalRelatorioPercentAdesao.show();
      });
    } else {
      this.modalRelatorioPercentAdesao.show();
    }
  }
  
  public cancelModalRelatorioPercentAdesao() {
    this.modalRelatorioPercentAdesao.hide();
  }
  
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');
    const row: IDictionary = form.value;
    if (!this.utilities.returnPrat(row.PRATO_INICIAL_REPORT, row.PRATO_FINAL_REPORT)
      || row.PRATO_FINAL_REPORT === row.PRATO_INICIAL_REPORT) {
      row.REPORT_NAME = row.REPORT.REPORT_NAME;
      row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
      
      row.CDFILCENTPME = row.REPORT.CDCENTRAL_REPORT;
      // row.NMCENTRAL = this.appController.getNmCentral();
      
      const maskPrat: string = (Config.masks.plateMask as PmeMask).pattern;
      row.CDPICTPRAT = this.utilities.formatReportMask(maskPrat);
      
      this.reportController.callGenerateReport(component, row, 'report/percentualAdesao');
    } else {
      this.utilities.showAlert('Alert', 'pme::pratoIniMaior', 'error');
    }
  }

  public verifyPratIniReport({ component }: IEventParam<SelectTree>) {
    const form = Metadata.getInstance<Form>('FORM');
    if (component.value !== null) {
      if (this.utilities.returnPrat(form.getChildInstance<SelectTree>('PRATO_INICIAL_REPORT').value, component.value)) {
        this.utilities.showAlert('Alert', 'pme::pratoIniMaior', 'error');
      }
    }
  }
  
  public filterAgregadaReport({ component }: IEventParam<Select>) {
    const central = Metadata.getInstance<Select>('CDCENTRAL_REPORT');
    
    component.datasource.filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: central.value.length > 0 ? central.value.join('_') : 'T',
      },
    ];
    component.datasource.get();
  }
  
  public clearFields() {
    const agregada = Metadata.getInstance<Select>('UNIDADE_AGREGADA_REPORT');
    const tpCardapio = Metadata.getInstance<Select>('TPCARD_REPORT');

    agregada.setValue('');
    tpCardapio.setValue('');
  }
}
